#include <jni.h>
#include <stdio.h>
#include <malloc.h>

#include "string.h"

#include "WCDMAttributeCreator.h"
#include "WCDMContentManager.h"

#ifdef __cplusplus
WCDMContentManager CM_SESSION;
#endif


/*
 * Class:     WCDMAttributeCreator
 * Method:    openConnection
 */
JNIEXPORT jlong JNICALL 
Java_WCDMAttributeCreator_openConnection(JNIEnv *env, 
                                         jobject obj, 
                                         jstring dbName, 
                                         jstring userId,
                                         jstring password)
{
#ifdef __cplusplus
   // Allocate UTF strings for the key and valueName
   const char *tempStr1 = env->GetStringUTFChars(dbName, 0);
   const char *tempStr2 = env->GetStringUTFChars(userId, 0);
   const char *tempStr3 = env->GetStringUTFChars(password, 0);
#else
   // Allocate UTF strings for the key and valueName
   const char *tempStr1 = (*env)->GetStringUTFChars(env, dbName, 0);
   const char *tempStr2 = (*env)->GetStringUTFChars(env, userId, 0);
   const char *tempStr3 = (*env)->GetStringUTFChars(env, password, 0);
#endif /* __cplusplus */

   //printf("[WCDMAttributeCreator::openConnection]: args(0)=[%s, %s, %s]\n\n", tempStr1, tempStr2, tempStr3);

   // Set variables
   char* pszDbName = (char*)alloca(strlen(tempStr1)+1);
   strcpy(pszDbName, tempStr1);
   char* pszUserId = (char*)alloca(strlen(tempStr2)+1);
   strcpy(pszUserId, tempStr2);
   char* pszPassword = (char*)alloca(strlen(tempStr3)+1);
   strcpy(pszPassword, tempStr3);

   //printf("[WCDMAttributeCreator::openConnection]: args(1)=[%s, %s, %s]\n\n", pszDbName, pszUserId, pszPassword);

   LONG lResult = 0;
#ifdef __cplusplus
   lResult = CM_SESSION.openConnection(pszDbName, pszUserId, pszPassword, "");
#else
#endif

   printf("[WCDMAttributeCreator::openConnection]: result=%i\n\n", lResult);

#ifdef __cplusplus
   // Release memory for UTF strings
   env->ReleaseStringUTFChars(dbName, 0);
   env->ReleaseStringUTFChars(userId, 0);
   env->ReleaseStringUTFChars(password, 0);
#else
   // Release memory for UTF strings
   (*env)->ReleaseStringUTFChars(env, dbName, 0);
   (*env)->ReleaseStringUTFChars(env, userId, 0);
   (*env)->ReleaseStringUTFChars(env, password, 0);
#endif /* __cplusplus */

   return (lResult);
}

/*
 * Class:     WCDMAttributeCreator
 * Method:    closeConnection
 */
JNIEXPORT void JNICALL 
Java_WCDMAttributeCreator_closeConnection(JNIEnv *env, 
                                          jobject obj)
{
#ifdef __cplusplus
   CM_SESSION.closeConnection();
#else
#endif
}

/*
 * Class:     WCDMAttributeCreator
 * Method:    createAttribute
 * Signature: (Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jlong JNICALL 
Java_WCDMAttributeCreator_createAttribute(JNIEnv *env, 
                                          jobject obj, 
                                          jstring name, 
                                          jshort  attrType,
                                          jshort  charType,
                                          jshort  minLen,
                                          jshort  maxLen)
{
#ifdef __cplusplus
   // Allocate UTF strings for the key and valueName
   const char *tempStr = env->GetStringUTFChars(name, 0);
#else
   // Allocate UTF strings for the key and valueName
   const char *tempStr = (*env)->GetStringUTFChars(env, name, 0);
#endif

   // Set attribute name
   char* pszName = (char*)alloca(strlen(tempStr)+1);
   strcpy(pszName, tempStr);

   //printf("[WCDMAttributeCreator::createAttribute]: name=%s\n", pszName);
   //printf("[WCDMAttributeCreator::createAttribute]: attrType=%i\n\n", attrType);
   //printf("[WCDMAttributeCreator::createAttribute]: attrType=%i\n\n", charType);
   //printf("[WCDMAttributeCreator::createAttribute]: attrType=%i\n\n", minLen);
   //printf("[WCDMAttributeCreator::createAttribute]: attrType=%i\n\n", maxLen);

   // Create attribute
   LONG lResult = 0;

#ifdef __cplusplus
   lResult = CM_SESSION.createAttribute(pszName, attrType, minLen, maxLen, charType);
#else
#endif

   printf("[WCDMAttributeCreator::createAttribute]: attrName=%s; result=%i\n\n", pszName, lResult);

#ifdef __cplusplus
   // Release memory for UTF strings
   env->ReleaseStringUTFChars(name, 0);
#else
   // Release memory for UTF strings
   (*env)->ReleaseStringUTFChars(env, name, 0);
#endif /* __cplusplus */

   return (lResult);
}

/*
 * Class:     WCDMAttributeCreator
 * Method:    getErrorString
 */
JNIEXPORT jstring JNICALL 
Java_WCDMAttributeCreator_getErrorString(JNIEnv *env, 
                                         jobject obj)
{
   char errStr[128];

   sprintf(errStr, "RESULT=%i; EX_RESULT=%i; EX_REASON=%i", CM_SESSION.getResult(), CM_SESSION.getExternalResult(), CM_SESSION.getExternalReason());

#ifdef __cplusplus
   // Return value as UTF string
   return (env->NewStringUTF(errStr));
#else
   // Return value as UTF string
   return ((*env)->NewStringUTF(env, errStr));
#endif /* __cplusplus */
}

